<?php

/**
 * EXHIBIT A. Common Public Attribution License Version 1.0
 * The contents of this file are subject to the Common Public Attribution License Version 1.0 (the “License”);
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.oxwall.org/license. The License is based on the Mozilla Public License Version 1.1
 * but Sections 14 and 15 have been added to cover use of software over a computer network and provide for
 * limited attribution for the Original Developer. In addition, Exhibit A has been modified to be consistent
 * with Exhibit B. Software distributed under the License is distributed on an “AS IS” basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the specific language
 * governing rights and limitations under the License. The Original Code is Oxwall software.
 * The Initial Developer of the Original Code is Oxwall Foundation (http://www.oxwall.org/foundation).
 * All portions of the code written by Oxwall Foundation are Copyright (c) 2011. All Rights Reserved.

 * EXHIBIT B. Attribution Information
 * Attribution Copyright Notice: Copyright 2011 Oxwall Foundation. All rights reserved.
 * Attribution Phrase (not exceeding 10 words): Powered by Oxwall community software
 * Attribution URL: http://www.oxwall.org/
 * Graphic Image as provided in the Covered Code.
 * Display of Attribution Information is required in Larger Works which are defined in the CPAL as a work
 * which combines Covered Code or portions thereof with code not governed by the terms of the CPAL.
 */

/**
 * Desc...
 *
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow_core
 * @since 1.0
 */
class OW_Response
{
    /**
     * HTTP Header constants
     */
    const HD_CACHE_CONTROL = 'Cache-Control';
    const HD_CNT_DISPOSITION = 'Content-Disposition';
    const HD_CNT_LENGTH = 'Content-Length';
    const HD_CONNECTION = 'Connection';
    const HD_PRAGMA = 'Pragma';
    const HD_CNT_TYPE = 'Content-Type';
    const HD_EXPIRES = 'Expires';
    const HD_LAST_MODIFIED = 'Last-Modified';
    const HD_LOCATION = 'Location';

    /**
     * Headers to send with response
     *
     * @var array
     */
    private $headers = array();
    /**
     * Document to send
     *
     * @var OW_Document
     */
    private $document;
    /**
     * Rendered markup
     *
     * @var string
     */
    private $markup = '';
    /**
     * Singleton instance.
     *
     * @var OW_Response
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return OW_Response
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Constructor.
     */
    private function __construct()
    {
        
    }

    /**
     * @return OW_Document
     */
    public function getDocument()
    {
        return $this->document;
    }

    /**
     * @param OW_Document $document
     */
    public function setDocument( OW_Document $document )
    {
        $this->document = $document;
    }

    /**
     * Adds headers to response.
     *
     * @param string $name
     * @param string $value
     */
    public function setHeader( $name, $value )
    {
        $this->headers[trim($name)] = trim($value);
    }

    /**
     * Clears all headers.
     */
    public function clearHeaders()
    {
        $this->headers = array();
    }

    /**
     * Sends all added headers.
     */
    public function sendHeaders()
    {
        if ( !headers_sent() )
        {
            foreach ( $this->headers as $headerName => $headerValue )
            {
                if ( substr(mb_strtolower($headerName), 0, 4) === 'http' )
                {
                    header($headerName . ' ' . $headerValue);
                }
                else if ( mb_strtolower($headerName) === 'status' )
                {
                    header(ucfirst(mb_strtolower($headerName)) . ': ' . $headerValue, null, (int) $headerValue);
                }
                else
                {
                    header($headerName . ':' . $headerValue);
                }
            }
        }
    }

    /**
     * @return string
     */
    public function getMarkup()
    {
        return $this->markup;
    }

    /**
     * @param string $markup
     */
    public function setMarkup( $markup )
    {
        $this->markup = $markup;
    }

    /**
     * Sends generated response
     *
     */
    public function respond()
    {
        $event = new OW_Event(OW_EventManager::ON_BEFORE_DOCUMENT_RENDER);
        OW::getEventManager()->trigger($event);
        if ( $this->document !== null )
        {
            $this->markup = $this->document->render();
        }

        $event = new OW_Event(OW_EventManager::ON_AFTER_DOCUMENT_RENDER);
        OW::getEventManager()->trigger($event);

        $this->sendHeaders();

        if ( OW::getRequest()->isAjax() )
        {
            exit();
        }

        if ( OW_PROFILER_ENABLE )
        {
            UTIL_Profiler::getInstance()->mark('final');
        }

        if ( OW_DEBUG_MODE )
        {
            echo ob_get_clean();
        }

        echo $this->markup;

        if ( OW_PROFILER_ENABLE )
        {
            echo('<div style="background:#fff;">' .
            'Queries on page = <span style="color:red;font-weight:bold;">' . OW::getDbo()->getQueryCount() . '</span>&nbsp;&nbsp;' .
            'Queries exec time = <span style="color:red;font-weight:bold;">' . sprintf('%.3f', OW::getDbo()->getTotalQueryExecTime()) . '</span>&nbsp;&nbsp;' .
            'Page build time = <span style="color:red;font-weight:bold;">' . sprintf('%.3f', UTIL_Profiler::getInstance()->getTotalTime()) . '</span></span>&nbsp;&nbsp;' .
            'Memory usage = <span style="color:red;font-weight:bold;">' . sprintf('%0.2f', memory_get_usage() / 1048576) . ' MB' . '</span></span>&nbsp;&nbsp;' .
            'Peak memory usage = <span style="color:red;font-weight:bold;">' . sprintf('%0.2f', memory_get_peak_usage() / 1048576) . ' MB' . '</span>' .
            '</div>'
            );

            $queryLog = OW::getDbo()->getQueryLog();

            printVar($queryLog);

            foreach ( $queryLog as $key => $queryItem )
            {
                if ( !strstr($queryItem['query'], 'ow_base_user ') )
                {
                    unset($queryLog[$key]);
                }
            }
            printVar(count($queryLog));
            printVar($queryLog);
        }

        $event = new OW_Event('core.exit');
        OW::getEventManager()->trigger($event);
    }
}