<?php

/**
 * EXHIBIT A. Common Public Attribution License Version 1.0
 * The contents of this file are subject to the Common Public Attribution License Version 1.0 (the “License”);
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.oxwall.org/license. The License is based on the Mozilla Public License Version 1.1
 * but Sections 14 and 15 have been added to cover use of software over a computer network and provide for
 * limited attribution for the Original Developer. In addition, Exhibit A has been modified to be consistent
 * with Exhibit B. Software distributed under the License is distributed on an “AS IS” basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the specific language
 * governing rights and limitations under the License. The Original Code is Oxwall software.
 * The Initial Developer of the Original Code is Oxwall Foundation (http://www.oxwall.org/foundation).
 * All portions of the code written by Oxwall Foundation are Copyright (c) 2011. All Rights Reserved.

 * EXHIBIT B. Attribution Information
 * Attribution Copyright Notice: Copyright 2011 Oxwall Foundation. All rights reserved.
 * Attribution Phrase (not exceeding 10 words): Powered by Oxwall community software
 * Attribution URL: http://www.oxwall.org/
 * Graphic Image as provided in the Covered Code.
 * Display of Attribution Information is required in Larger Works which are defined in the CPAL as a work
 * which combines Covered Code or portions thereof with code not governed by the terms of the CPAL.
 */

/**
 * Class is responsible for...    
 * 
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow_core
 * @since 1.0
 */
class OW_ViewRenderer
{
    /**
     * @var OW_Smarty
     */
    private $smarty;
    /**
     * Singleton instance.
     * 
     * @var OW_ViewRenderer
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return OW_ViewRenderer
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Constructor.
     */
    private function __construct()
    {
        $this->smarty = new OW_Smarty();
    }

    /**
     * Assigns list of values to template vars by reference.
     *
     * @param array $vars
     */
    public function assignVars( $vars )
    {
        foreach ( $vars as $key => $value )
        {
            $this->smarty->assign_by_ref($key, $vars[$key]);
        }
    }

    /**
     * Assigns value to template var by reference.
     *
     * @param string $key
     * @param mixed $value
     */
    public function assignVar( $key, $value )
    {
        $this->smarty->assign_by_ref($key, $value);
    }

    /**
     * Renders template using assigned vars and returns generated markup.
     *
     * @param string $template
     * @return string
     */
    public function renderTemplate( $template )
    {
        return $this->smarty->fetch($template);
    }

    /**
     * Returns assigned var value for provided var name.
     *
     * @param string $varName
     * @return mixed
     */
    public function getAssignedVar( $varName )
    {
        return $this->smarty->get_template_vars($varName);
    }

    /**
     * Returns list of assigned var values.
     *
     * @return array
     */
    public function getAllAssignedVars()
    {
        return $this->smarty->get_template_vars();
    }

    /**
     * Deletes all assigned template vars.
     */
    public function clearAssignedVars()
    {
        $this->smarty->clear_all_assign();
    }

    /**
     *
     * @param string $varName
     */
    public function clearAssignedVar( $varName )
    {
        $this->smarty->clear_assign($varName);
    }

    /**
     * Adds custom function for template.
     *
     * @param string $functionName
     * @param callback $function
     */
    public function registerFunction( $functionName, $function )
    {
        $this->smarty->register_function($functionName, $function);
    }

    /**
     * Removes custom function.
     *
     * @param string $functionName
     */
    public function unregisterFunction( $functionName )
    {
        $this->smarty->unregister_function($functionName);
    }

    /**
     * Adds custom block function for template.
     *
     * @param string $blockName
     * @param callback $blockFunctions
     */
    public function registerBlock( $blockName, $blockFunctions )
    {
        $this->smarty->register_block($blockName, $blockFunctions);
    }

    /**
     * Removes block function.
     *
     * @param string $blockName
     */
    public function unregisterBlock( $blockName )
    {
        $this->smarty->unregister_block($blockName);
    }

    /**
     * Changes left and right delimiters.
     *
     * @param string $leftDelimiter
     * @param string $rightDelimiter
     */
    public function setDelimiters( $leftDelimiter = '{', $rightDelimiter = '}' )
    {
        $this->smarty->left_delimiter = $leftDelimiter;
        $this->smarty->right_delimiter = $rightDelimiter;
    }

    /**
     * Clears compiled templates.
     */
    public function clearCompiledTpl()
    {
        $this->smarty->clear_compiled_tpl();
    }
}